@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <signal.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -310,18 +311,12 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
         return PAM_IGNORE;
     }
 
-    char *key = malloc(KWALLET_PAM_KEYSIZE);
-    if (!key || kwallet_hash(pamh, password, userInfo, key) != 0) {
-        free(key);
-        pam_syslog(pamh, LOG_ERR, "%s: Fail into creating the hash", logPrefix);
-        return PAM_IGNORE;
-    }
-
+    char *key = strdup(password);
     result = pam_set_data(pamh, kwalletPamDataKey, key, cleanup_free);
 
     if (result != PAM_SUCCESS) {
         free(key);
-        pam_syslog(pamh, LOG_ERR, "%s: Impossible to store the hashed password: %s", logPrefix
+        pam_syslog(pamh, LOG_ERR, "%s: Impossible to store the password: %s", logPrefix
             , pam_strerror(pamh, result));
         return PAM_IGNORE;
     }
@@ -361,12 +356,13 @@ static int drop_privileges(struct passwd *userInfo)
 static void execute_kwallet(pam_handle_t *pamh, struct passwd *userInfo, int toWalletPipe[2], char *fullSocket)
 {
     //In the child pam_syslog does not work, using syslog directly
+
     //keep stderr open so socket doesn't returns us that fd
     int x = 3;
-    //Close fd that are not of interest of kwallet
+    //Set FD_CLOEXEC on fd that are not of interest of kwallet
     for (; x < 64; ++x) {
         if (x != toWalletPipe[0]) {
-            close (x);
+            fcntl(x, F_SETFD, FD_CLOEXEC);
         }
     }
 
@@ -574,15 +570,22 @@ PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, cons
         return PAM_IGNORE;
     }
 
-    const char *kwalletKey;
-    result = pam_get_data(pamh, kwalletPamDataKey, (const void **)&kwalletKey);
+    char *password;
+    result = pam_get_data(pamh, kwalletPamDataKey, (const void **)&password);
 
     if (result != PAM_SUCCESS) {
         pam_syslog(pamh, LOG_INFO, "%s: open_session called without %s", logPrefix, kwalletPamDataKey);
         return PAM_SUCCESS;//We will wait for pam_sm_authenticate
     }
 
-    start_kwallet(pamh, userInfo, kwalletKey);
+    char *key = malloc(KWALLET_PAM_KEYSIZE);
+    if (!key || kwallet_hash(pamh, password, userInfo, key) != 0) {
+        free(key);
+        pam_syslog(pamh, LOG_ERR, "%s: Fail into creating the hash", logPrefix);
+        return PAM_IGNORE;
+    }
+
+    start_kwallet(pamh, userInfo, key);
 
     return PAM_SUCCESS;
 }
